import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { Button, notification, Modal } from 'antd'
import moment from 'moment'

import MutilForm from './mutilform'
import Utils from '@/utils/utils.js'
import Api from '@/api'
import './index.scss'

class CreateFunc extends Component {
  static propTypes = {
    dict: PropTypes.object,  // 字典项
    trigger: PropTypes.func
  }

  state = {
    type: '',
    param: null,
    loading: false,
    visible: false,
    config: null,
    btn: null
  }

  /**
   * @description 触发创建读数据接口
   * @param { object } menu    菜单类型（三级菜单或标签页）、菜单ID、菜单参数、菜单名称
   * @param { object } config  菜单配置信息
   */
  triggerOutInterface = (menu, config) => {
    let _search = []
    let _index = 1

    if (menu.type !== 'main') {
      // 子表页面增加BID字段，数据源中使用了@BID@时，BID类型为必传
      if (config.setting.dataresource && /@BID@/ig.test(config.setting.dataresource)) {
        _search.push(`select 'BID' as searchfield,'BID' as label,'0' as Sort,'' as defaultvalue,'required' as DefaultType`)
      } else {
        _search.push(`select 'BID' as searchfield,'BID' as label,'0' as Sort,'' as defaultvalue,'' as DefaultType`)
      }
    }

    let _fieldMap = new Map()

    config.search.forEach(item => {
      let fields = [item.field]
      let _val = item.initval

      if (item.type === 'text') {
        fields = item.field.split(',')
      }
      if (/^date/ig.test(item.type)) {
        _val = ''
      }

      if (item.type === 'datemonth' || item.type === 'dateweek' || item.type === 'daterange') {
        fields = [item.field, item.field]
      }

      fields.forEach(cell => {
        let _field = cell
        if (_fieldMap.has(cell)) {
          _field = _field + '1'
        }

        _fieldMap.set(cell, true)

        _search.push(`select '${_field}' as searchfield,'${item.label}' as label,'${_index}' as Sort,'${_val}' as defaultvalue,'${item.required === 'true' ? 'required' : ''}' as DefaultType`)
        _index++
      })
    })

    let searchText = []
    _fieldMap = new Map()

    config.search.forEach((item, i) => {
      if (item.type === 'text') {
        let str = item.match === '=' ? '' : '\'%\''
        let _fields_ = item.field.split(',').map(field => { // 综合搜索，所字段拼接

          return field + ' ' + item.match + ` ${str ? str + '+' : ''}@` + field + `@${str ? '+' + str : ''}`
        })

        if (_fields_.length === 1) {
          searchText.push(_fields_[0])
        } else {
          searchText.push('(' + _fields_.join(' OR ') + ')')
        }
      } else if (item.type === 'select') {

        searchText.push(item.field + ' ' + item.match + ' @' + item.field + '@')
      } else if (item.type === 'multiselect') {
        
        searchText.push(`@${item.field}@ ` + item.match + ' \'%\'+' + item.field + '+\'%\'')
      } else if (item.type === 'date') {
        let _field = item.field
        if (_fieldMap.has(item.field)) {
          _field = _field + '1'
        }

        _fieldMap.set(item.field, true)

        searchText.push(item.field + ' ' + item.match + ' @' + _field + '@')
      } else if (item.type === 'datemonth' || item.type === 'dateweek' || item.type === 'daterange') {

        searchText.push('(' + item.field + ' >= @' + item.field + '@ AND ' + item.field + ' < @' + item.field + '1@)')
      } else {
        searchText.push(item.field + ' ' + item.match + ' @' + item.field + '@')
      }
    })

    let param = {
      func: 's_get_para_for_out',
      Menuid: menu.MenuID,
      Menuname: menu.menuName,
      Menuno: menu.menuNo,
      PageType: 'Y',
      Ltextsearchparam: _search.join(' union all '),
      AppendWhere: config.setting.queryType === 'query' ? searchText.join(' AND ') : '',
      WhereType: config.setting.queryType === 'statistics' ? 'Statistics' : 'query',
      custom_script: config.setting.customScript || '',
      default_sql: config.setting.default || 'true',
      OrderCol: config.setting.order
    }

    param.Ltextsearchparam = Utils.formatOptions(param.Ltextsearchparam)
    param.AppendWhere = Utils.formatOptions(param.AppendWhere)
    param.custom_script = Utils.formatOptions(param.custom_script)

    this.setState({
      type: 'out',
      param: param,
      config: config,
      visible: true,
      formlist: [{
        type: 'text',
        key: 'Menuname',
        label: '接口名称',
        initval: menu.menuName,
        required: true
      }, {
        type: 'text',
        key: 'Menuno',
        label: '接口函数',
        initval: menu.menuNo,
        required: true
      }, {
        type: 'radio',
        key: 'PageType',
        label: '是否分页',
        initval: 'Y',
        required: true,
        options: [{
          value: 'Y',
          text: this.props.dict['header.form.true']
        }, {
          value: 'N',
          text: this.props.dict['header.form.false']
        }]
      }, {
        type: 'radio',
        key: 'hidden',
        label: '返回隐藏字段',
        initval: 'N',
        required: true,
        options: [{
          value: 'Y',
          text: this.props.dict['header.form.true']
        }, {
          value: 'N',
          text: this.props.dict['header.form.false']
        }]
      }]
    })
  }

  confirmInterface = () => {
    const { type, param, btn, config } = this.state

    if (type === 'out') {
      this.FormRef.handleConfirm().then(res => {
  
        this.setState({
          type: '',
          loading: true,
          visible: false,
          config: null,
          param: null
        })
    
        this.createExecOutInterface({...param, ...res}, config)
      })
    } else {
      this.FormRef.handleConfirm().then(res => {
  
        this.setState({
          type: '',
          btn: null,
          config: null,
          loading: true,
          visible: false,
          param: null
        })
    
        this.createBtnInterfaceExec({...param, ...res}, config, btn)
      })
    }
  }
  
  createExecOutInterface = (param, config) => {
    let _grid = []
    let _fields = []
    let _index = 1

    config.columns.forEach(item => {
      if (!item.field || item.type === 'colspan') return
      if (param.hidden !== 'Y' && item.Hide === 'true') return

      let _type = `nvarchar(${item.fieldlength || 50})`

      if (item.type === 'number') {
        _type = `decimal(18,${item.decimal ? item.decimal : 0})`
      } else if (item.type === 'picture' || item.type === 'textarea') {
        _type = `nvarchar(${item.fieldlength || 512})`
      }

      _grid.push(item.field)
      _fields.push(`select '${item.field}' as gridfield,'${_type}' as fieldtype,'${item.label}' as label,'${_index}' as Sort`)
      _index++
    })

    param.Ltextgrid = _grid.join(',')
    param.Ltextgridparam = _fields.join(' union all ')
    param.Ltextgridparam = Utils.formatOptions(param.Ltextgridparam)
    param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
    param.secretkey = Utils.encrypt(param.Ltextgridparam, param.timestamp)

    delete param.hidden

    let _mainParam = JSON.parse(JSON.stringify(param))

    new Promise(resolve => {
      // 获取云端存储过程信息
      Api.getLocalConfig(param).then(result => {
        if (!result.status) {
          notification.warning({
            top: 92,
            message: result.message,
            duration: 5
          })
          resolve(false)
        } else {
          resolve(true)
        }
      })
    }).then(res => {
      if (res === false) return res

      if (window.GLOB.mainSystemApi) {
        _mainParam.rduri = window.GLOB.mainSystemApi

        return Api.getLocalConfig(_mainParam)
      }
      return 'success'
    }).then(result => {
      if (result === false || result === 'success') return result
      
      if (!result.status) {
        notification.warning({
          top: 92,
          message: result.message,
          duration: 5
        })
        return false
      } else {
        return 'success'
      }
    }).then(res => {
      // 结果处理
      if (res !== false) {
        notification.success({
          top: 92,
          message: '创建成功',
          duration: 2
        })
      }

      this.setState({
        loading: false
      })
    })
  }

  /**
   * @description 触发创建写入接口（按钮）
   */
  triggerInInterface = (btn, config, menu) => {
    if (!['pop', 'exec', 'prompt'].includes(btn.OpenType) || btn.funcType || btn.intertype !== 'inner' || btn.innerFunc ) {
      notification.warning({
        top: 92,
        message: '打开方式为 弹窗（表单）、提示框或直接执行，且使用系统函数时，才可以创建接口！',
        duration: 5
      })
      return
    } else if (btn.Ot === 'requiredOnce') {
      notification.warning({
        top: 92,
        message: '多行拼接时，不支持创建接口！',
        duration: 5
      })
      return
    }

    let param = {
      func: 's_get_para_for_in',
      Menuid: btn.uuid,
      KunName: menu.menuName + '-' + btn.label,
      KunNo: menu.menuNo + '_' + btn.sqlType,
      Ltextgridparam: '',
      Ltexttableparam: '',
      Ltext: '',
      menuType: menu.type
    }

    this.setState({
      type: 'inner',
      param: param,
      visible: true,
      btn: JSON.parse(JSON.stringify(btn)),
      config: JSON.parse(JSON.stringify(config)),
      formlist: [{
        type: 'text',
        key: 'KunName',
        label: '接口名称',
        initval: param.KunName,
        required: true
      }, {
        type: 'text',
        key: 'KunNo',
        label: '接口函数',
        initval: param.KunNo,
        required: true
      }, {
        type: 'radio',
        key: 'TryType',
        label: '事务',
        initval: 'N',
        required: true,
        options: [{
          value: 'Y',
          text: '需要'
        }, {
          value: 'N',
          text: '不需要'
        }]
      }, {
        type: 'radio',
        key: 'Return',
        label: '回执',
        initval: 'N',
        required: true,
        options: [{
          value: 'Y',
          text: '需要'
        }, {
          value: 'N',
          text: '不需要'
        }]
      }]
    })
  }

  /**
   * @description 执行接口创建
   */
  createBtnInterfaceExec = (param, config, btn) => {
    let formlist = []
    let receipt = param.Return === 'Y'
    let _mainParam = null

    delete param.Return

    new Promise(resolve => {
      if (btn.OpenType === 'pop') {
        Api.getSystemConfig({
          func: 'sPC_Get_LongParam',
          MenuID: btn.uuid
        }).then(result => {
          if (result.status && result.LongParam) {
            let _LongParam = ''
  
            // 解析配置，修改模态框标题名称
            if (result.LongParam) {
              try {
                _LongParam = JSON.parse(window.decodeURIComponent(window.atob(result.LongParam)))
              } catch (e) {
                console.warn('Parse Failure')
                _LongParam = ''
              }
            }
            let fields = []

            if (_LongParam && _LongParam.type === 'Modal') {
              if (_LongParam.groups.length > 0) {
                _LongParam.groups.forEach(group => {
                  fields = [...fields, ...group.sublist]
                })
              } else {
                fields = _LongParam.fields
              }
            }

            if (fields && fields.length > 0) {
              formlist = fields.map(cell => {
                let _fieldlen = cell.fieldlength || 50

                if (cell.type === 'textarea' || cell.type === 'fileupload' || cell.type === 'multiselect') {
                  _fieldlen = cell.fieldlength || 512
                } else if (cell.type === 'number') {
                  _fieldlen = cell.decimal ? cell.decimal : 0
                }

                let _field = {
                  type: cell.type,
                  label: cell.label,
                  readonly: cell.readonly,
                  readin: cell.readin !== 'false',
                  fieldlen: _fieldlen,
                  key: cell.field,
                  required: cell.required === 'true' ? 'required' : '',
                  value: cell.initval || ''
                }

                let _fieldtype = `nvarchar(${_fieldlen})`

                if (_field.type.match(/date/ig)) {
                  _fieldtype = 'datetime'
                } else if (_field.type === 'number') {
                  _fieldtype = `decimal(18,${_fieldlen})`
                }

                _field.fieldtype = _fieldtype

                if (cell.type === 'funcvar') {
                  _field.readin = false
                  _field.value = ''
                }

                return _field
              })
            }
            resolve(true)
          } else if (!result.status) {
            notification.warning({
              top: 92,
              message: result.message,
              duration: 5
            })
            resolve(false)
          } else {
            notification.warning({
              top: 92,
              message: '请完善表单信息！',
              duration: 5
            })
            resolve(false)
          }
        })
      }
    }).then(res => {
      if (res === false) return res
      
      let _keys = []
      param.Ltexttableparam = formlist.map((item, index) => {
        _keys.push(item.key.toLowerCase())

        return `select '${item.key}' as searchfield,'${item.label}' as label,'${index + 2}' as Sort, '${item.fieldtype}' as fieldtype,'${item.required}' as requiredtype,'${item.value}' as defaultvalue`
      })

      if (param.menuType !== 'main' && !_keys.includes('bid')) {
        _keys.push('bid')
        param.Ltexttableparam.unshift(`select 'BID' as searchfield,'BID' as label,'0' as Sort,'nvarchar(50)' as fieldtype,'required' as requiredtype,'' as defaultvalue`)
      }

      if (btn.Ot !== 'notRequired' && !_keys.includes('id')) {
        param.Ltexttableparam.unshift(`select 'ID' as searchfield,'ID' as label,'1' as Sort,'nvarchar(50)' as fieldtype,'required' as requiredtype,'' as defaultvalue`)
      }

      param.Ltexttableparam = param.Ltexttableparam.join(' union all ')
      
      if (receipt) {
        let _keys = []

        param.Ltextgridparam = formlist.map((item, index) => {
          _keys.push(item.key.toLowerCase())

          return `select '${item.key}' as gridfield, '${item.fieldtype}' as fieldtype,'${item.label}' as label,'${index + 2}' as Sort`
        })

        if (param.menuType !== 'main' && !_keys.includes('bid')) {
          _keys.push('bid')
          param.Ltextgridparam.unshift(`select 'BID' as gridfield,'nvarchar(50)' as fieldtype,'BID' as label,'0' as Sort`)
        }

        if (!_keys.includes(config.setting.primaryKey.toLowerCase())) {
          _keys.push(config.setting.primaryKey.toLowerCase())
          param.Ltextgridparam.unshift(`select '${config.setting.primaryKey}' as gridfield,'nvarchar(50)' as fieldtype,'${config.setting.primaryKey}' as label,'1' as Sort`)
        }

        if (btn.verify && btn.verify.voucher && btn.verify.voucher.enabled && btn.Ot !== 'notRequired') {
          if (!_keys.includes('bvoucher')) {
            param.Ltextgridparam.unshift(`select 'bvoucher' as gridfield,'nvarchar(50)' as fieldtype,'bvoucher' as label,'30' as Sort`)
          }
          if (!_keys.includes('fibvoucherdate')) {
            param.Ltextgridparam.unshift(`select 'fibvoucherdate' as gridfield,'nvarchar(50)' as fieldtype,'fibvoucherdate' as label,'31' as Sort`)
          }
          if (!_keys.includes('fiyear')) {
            param.Ltextgridparam.unshift(`select 'fiyear' as gridfield,'nvarchar(50)' as fieldtype,'fiyear' as label,'32' as Sort`)
          }
        }

        param.Ltextgridparam = param.Ltextgridparam.join(' union all ')
      }

      let BID = param.menuType !== 'main' ? '@BID@' : ''

      param.Ltextgridparam = Utils.formatOptions(param.Ltextgridparam)
      param.Ltexttableparam = Utils.formatOptions(param.Ltexttableparam)

      param.Ltext = this.getInterfaceSysDefaultSql(btn, config, formlist, receipt, BID)
      param.Ltext = Utils.formatOptions(param.Ltext)
      param.timestamp = moment().format('YYYY-MM-DD HH:mm:ss') + '.000'
      param.secretkey = Utils.encrypt(param.Ltext, param.timestamp)
      
      delete param.menuType

      _mainParam = JSON.parse(JSON.stringify(param))

      return Api.getLocalConfig(param)
    }).then(res => {
      if (res === false) return res

      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return false
      } else {
        return true
      }
    }).then(res => {
      if (res === false) return res

      if (window.GLOB.mainSystemApi) {
        _mainParam.rduri = window.GLOB.mainSystemApi

        return Api.getLocalConfig(_mainParam)
      }
      return 'success'
    }).then(res => {
      if (res === false || res === 'success') return res

      if (!res.status) {
        notification.warning({
          top: 92,
          message: res.message,
          duration: 5
        })
        return false
      } else {
        return 'success'
      }
    }).then(res => {

      if (res === 'success') {
        notification.success({
          top: 92,
          message: '创建成功',
          duration: 2
        })
      }
      this.setState({
        loading: false
      })
    })
  }

  getInterfaceSysDefaultSql = (btn, config, formlist, receipt, BID) => {
    let setting = config.setting
    let primaryKey = setting.primaryKey || 'id'
    let columns = config.columns.filter(col => !!col.field)
    let verify = btn.verify || {}
    // 需要声明的变量集
    let _vars = ['tbid', 'errorcode', 'retmsg', 'billcode', 'bvoucher', 'fibvoucherdate', 'fiyear', 'username', 'fullname', 'modulardetailcode']

    // 系统变量声明与设置初始值
    let _sql = `Declare @tbid nvarchar(50),@ErrorCode nvarchar(50),@retmsg nvarchar(4000),@BillCode nvarchar(50),@BVoucher nvarchar(50),@FIBVoucherDate nvarchar(50), @FiYear nvarchar(50), @UserName nvarchar(50),@FullName nvarchar(50),@ModularDetailCode nvarchar(50)
      `

    let _initvars = [] // 已赋值字段集
    let _initfields = []
    let _initcolumnfields = []
    let _declarefields = []

    // 获取字段键值对
    formlist.forEach(form => {
      let _key = form.key.toLowerCase()

      if (!_initvars.includes(_key)) {
        _initvars.push(_key)

        _initfields.push(`@${_key}=@${_key}@`)
      }
      
      if (!_vars.includes(_key)) {
        _vars.push(_key)

        let _type = `nvarchar(${form.fieldlen})`

        if (form.type.match(/date/ig)) {
          _type = 'datetime'
        } else if (form.type === 'number') {
          _type = `decimal(18,${form.fieldlen})`
        }

        _declarefields.push(`@${_key} ${_type}`)
      }
    })

    // 添加数据中字段，表单值优先(按钮不选行时跳过)
    if (btn.Ot !== 'notRequired') {
      columns.forEach(col => {
        let _key = col.field.toLowerCase()

        if (!_initvars.includes(_key)) {
          _initvars.push(_key)

          _initcolumnfields.push(`@${_key}=${_key}`)
        }
        
        if (!_vars.includes(_key)) {
          _vars.push(_key)

          let _type = `nvarchar(${col.fieldlength || 50})`

          if (col.type === 'number') {
            let _length = col.decimal ? col.decimal : 0
            _type = `decimal(18,${_length})`
          } else if (col.type === 'picture' || col.type === 'textarea') {
            _type = `nvarchar(${col.fieldlength || 512})`
          }

          _declarefields.push(`@${_key} ${_type}`)
        }
      })
    }

    // 变量声明
    _declarefields = _declarefields.join(',')
    if (_declarefields) {
      _sql += `,${_declarefields}
        `
    }

    // 变量赋值
    if (_initfields.length > 0) {
      _sql += `select ${_initfields.join(',')}
        `
    }

    if (_initcolumnfields.length > 0) {
      _sql += `select ${_initcolumnfields.join(',')} from ${setting.dataresource} where ${primaryKey}=@ID@
        `
    }
    
    // 去除禁用的验证
    if (verify.contrasts) {
      verify.contrasts = verify.contrasts.filter(item => item.status !== 'false')
    }
    if (verify.uniques) {
      verify.uniques = verify.uniques.filter(item => item.status !== 'false')
    }
    if (verify.customverifys) {
      verify.customverifys = verify.customverifys.filter(item => item.status !== 'false')
    }
    if (verify.billcodes) {
      verify.billcodes = verify.billcodes.filter(item => item.status !== 'false')
    }
    if (verify.scripts) {
      verify.scripts = verify.scripts.filter(item => item.status !== 'false')
    }

    // 初始化凭证及用户信息字段
    _sql += `select @BVoucher='',@FIBVoucherDate='',@FiYear='',@ErrorCode='',@retmsg=''
      `

    // 启用账期验证
    if (verify.accountdate === 'true') {
      _sql += `exec s_FIBVoucherDateCheck @ErrorCode=@ErrorCode OUTPUT,@retmsg=@retmsg OUTPUT
        if @ErrorCode!=''
          GOTO aaa
        `
    }

    // 失效验证，添加数据时不用
    if (btn.sqlType !== 'insert' && verify.invalid === 'true') {
      let datasource = setting.dataresource
      if (/\s/.test(datasource)) { // 拼接别名
        datasource = '(' + datasource + ') tb'
      }

      _sql += `select @tbid='', @ErrorCode='',@retmsg=''
        select @tbid=${primaryKey} from ${datasource} where ${primaryKey} =@${primaryKey}@
        If @tbid=''
        Begin
          select @ErrorCode='E',@retmsg='数据已失效'
          goto aaa
        end
        `
    }

    // 比较验证
    if (verify.contrasts && verify.contrasts.length > 0) {
      verify.contrasts.forEach(item => {
        _sql += `If ${item.frontfield} ${item.operator} ${item.backfield}
          Begin
            select @ErrorCode='${item.errorCode}',@retmsg='${item.errmsg}'
              goto aaa
          end
          `
      })
    }

    // 唯一性验证，必须存在表单（表单存在时，主键均为单值）,必须填写数据源
    if (formlist.length > 0 && verify.uniques && verify.uniques.length > 0) {
      verify.uniques.forEach(item => {
        let _fieldValue = []                     // 表单键值对field=value
        let _value = []                          // 表单值，用于错误提示
        let _labels = item.fieldlabel.split(',') // 表单提示文字
        let arr = [] // 验证主键

        item.field.split(',').forEach((_field, index) => {
          _fieldValue.push(`${_field}=@${_field}@`)
          _value.push(`${_labels[index] || ''}：'+@${_field}@+'`)
          arr.push(_field.toLowerCase())
        })

        let _verifyType = ''
        if (item.verifyType === 'logic') {
          _verifyType = ' and deleted=0'
        }

        if (!arr.includes(primaryKey.toLowerCase())) {
          _fieldValue.push(`${primaryKey} !=@${primaryKey}@`)
        }

        _sql += `select @tbid='', @ErrorCode='',@retmsg=''
          select @tbid='X' from ${btn.sql} where ${_fieldValue.join(' and ')}${_verifyType}
          If @tbid!=''
          Begin
            select @ErrorCode='${item.errorCode}',@retmsg='${_value.join(', ')} 已存在'
            goto aaa
          end
          `
      })
    }
    
    // 自定义验证
    if (verify.customverifys && verify.customverifys.length > 0) {
      verify.customverifys.forEach(item => {
        _sql += `select @tbid='', @ErrorCode='',@retmsg=''
          select top 1 @tbid='X' from (${item.sql}) a
          If @tbid ${item.result === 'true' ? '!=' : '='}''
          Begin
            select @ErrorCode='${item.errorCode}',@retmsg='${item.errmsg}'
            goto aaa
          end
          `
      })
    }

    // 单号生成，使用上级id（BID）或列表数据，声明变量（检验）
    if (verify.billcodes && verify.billcodes.length > 0) {
      verify.billcodes.forEach(item => {
        let _ModularDetailCode = ''
        let _lpline = ''
        if (item.TypeCharOne === 'Lp') {
          if (item.linkField === 'BID' && BID) { // 替换bid
            _lpline = `set @ModularDetailCode= 'Lp'+ right('${btn.uuid}'+@BID@,48)`
          } else {
            _lpline = `set @ModularDetailCode= 'Lp'+ right('${btn.uuid}'+@${item.linkField}@,48)`
          }
          _ModularDetailCode = '@ModularDetailCode'
        } else if (item.TypeCharOne === 'BN') {

          _ModularDetailCode = `'${item.TypeCharOne + '@' + item.linkField}@'`
        } else {
          _ModularDetailCode = `'${item.ModularDetailCode}'`
        }

        let _declare = ''
        let _key = item.field.toLowerCase()

        if (!_vars.includes(_key)) {
          _declare = `Declare @${_key} nvarchar(50)`
          _vars.push(_key)
        }

        _sql += `${_declare}
          select @BillCode='', @${_key}='', @ModularDetailCode=''
          ${_lpline}
          exec s_get_BillCode
            @ModularDetailCode=${_ModularDetailCode},
            @Type=${item.Type},
            @TypeCharOne='${item.TypeCharOne}',
            @TypeCharTwo ='${item.TypeCharTwo}',
            @BillCode =@BillCode output,
            @ErrorCode =@ErrorCode output, 
            @retmsg=@retmsg output
          if @ErrorCode!=''
            goto aaa
          set @${_key}=@BillCode
          `
      })
    }

    let hasvoucher = false

    // 凭证-显示列中选取,必须选行
    if (verify.voucher && verify.voucher.enabled && btn.Ot !== 'notRequired') {
      let _voucher = verify.voucher

      hasvoucher = true

      _sql += `exec s_BVoucher_Create
          @Bill =@${_voucher.linkField}@',
          @BVoucherType ='${_voucher.BVoucherType}',
          @VoucherTypeOne ='${_voucher.VoucherTypeOne}',
          @VoucherTypeTwo ='${_voucher.VoucherTypeTwo}',
          @Type =${_voucher.Type},
          @UserID=@UserID@,
          @Username=@Username,
          @FullName=@FullName,
          @BVoucher =@BVoucher OUTPUT ,
          @FIBVoucherDate =@FIBVoucherDate OUTPUT ,
          @FiYear =@FiYear OUTPUT ,
          @ErrorCode =@ErrorCode OUTPUT, 
          @retmsg=@retmsg OUTPUT
        if @ErrorCode!=''
          GOTO aaa
        `
    }

    let _actionType = null
    let receiptKeys = [] // 回执字段

    if (verify.default !== 'false') { // 判断是否使用默认sql
      _actionType = btn.sqlType
    }
    
    formlist.forEach(item => {
      receiptKeys.push(item.key.toLowerCase())
    })

    if (!receiptKeys.includes(primaryKey.toLowerCase())) {
      receiptKeys.push(primaryKey.toLowerCase())
    }

    if (!receiptKeys.includes('bid') && BID) {
      receiptKeys.push('bid')
    }

    // 用于取用户信息
    let _user = `select @UserName=UserName,@FullName=FullName from SUsers where UID=@UserID@
      `

    // 添加、修改、逻辑删除、物理删除
    if (_actionType === 'insert') {
      let keys = []
      let values = []

      formlist.forEach(item => {
        keys.push(item.key.toLowerCase())
        values.push('@' + item.key + '@')
      })

      if (!keys.includes(primaryKey.toLowerCase())) {
        keys.push(primaryKey.toLowerCase())
        values.push(`@${primaryKey}@`)
      }

      if (!keys.includes('createuserid')) {
        keys.push('createuserid')
        values.push('@userid@')
      }
      if (!keys.includes('createuser')) {
        keys.push('createuser')
        values.push('@username')
      }
      if (!keys.includes('createstaff')) {
        keys.push('createstaff')
        values.push('@fullname')
      }
      if (!keys.includes('bid') && BID) {
        keys.push('bid')
        values.push('@BID@')
      }

      keys = keys.join(',')
      values = values.join(',')
      _sql += _user
      _sql += `insert into ${btn.sql} (${keys}) select ${values};`
    } else if (_actionType === 'update') {
      let _form = []
      let _arr = []

      formlist.forEach(item => {
        _arr.push(item.key.toLowerCase())

        _form.push(item.key + `=@${item.key}@`)
      })
      
      if (!_arr.includes('modifydate')) {
        _form.push('modifydate=getdate()')
      }
      if (!_arr.includes('modifyuserid')) {
        _form.push('modifyuserid=@userid@')
      }
      if (hasvoucher) {
        if (!_arr.includes('bvoucher')) {
          _arr.push('bvoucher')
          receiptKeys.push('bvoucher')
          _form.push('BVoucher=@BVoucher')
        }
        if (!_arr.includes('fibvoucherdate')) {
          _arr.push('fibvoucherdate')
          receiptKeys.push('fibvoucherdate')
          _form.push('FIBVoucherDate=@FIBVoucherDate')
        }
        if (!_arr.includes('fiyear')) {
          _arr.push('fiyear')
          receiptKeys.push('fiyear')
          _form.push('FiYear=@FiYear')
        }
      }

      if (!_arr.includes(primaryKey)) {
        _arr.push(primaryKey)
      }

      _form = _form.join(',')
      _sql += `update ${btn.sql} set ${_form} where ${primaryKey}=@${primaryKey}@;`
    } else if (_actionType === 'LogicDelete') { // 逻辑删除
      _sql += `update ${btn.sql} set deleted=1,modifydate=getdate(),modifyuserid=@userid@ where ${primaryKey}=@${primaryKey}@;`
      
    } else if (_actionType === 'delete') {      // 物理删除
      let _msg = ''
      if (columns.length > 0) {
        let _index = 0
        columns.forEach(col => {
          if (col.Hide !== 'true' && _index < 4) {
            _msg += col.label + `=@${col.field}@,`
            _index++
          }
        })
      }
      _sql += _user
      _sql += `insert into snote (remark,createuserid,CreateUser,CreateStaff) select '删除表:${btn.sql} 数据: ${_msg}${primaryKey}='+@${primaryKey}@,@userid@,@username,@fullname delete ${btn.sql} where ${primaryKey}=@${primaryKey}@;`
    }

    // 拼接自定义脚本
    if (verify.scripts && verify.scripts.length > 0) {
      let _scripts = ''
      verify.scripts.forEach(item => {
        _scripts += `
        ${item.sql}`
      })
      _sql += `${_scripts}`
    }

    let _ltext = ''
    if (receipt) {
      _ltext = `select obj_name='data',prm_field='${receiptKeys.join(',')}',str_field='',
			arr_field='',tabid='',parid='',sub_name='',sub_field=''
      select ${receiptKeys.map(key => `@${key}@ as ${key}`).join(',')}
      `
      _ltext = Utils.formatOptions(_ltext)
    }

    _sql += `
      aaa: select @ErrorCode as ErrorCode,@retmsg as retmsg,'${_ltext}' as Ltext`

    return _sql
  }

  render() {

    return (
      <div style={{display: 'inline-block', marginRight: '8px'}}>
        <Button
          className="mk-btn mk-green"
          onClick={this.props.trigger}
          loading={this.state.loading}
        >
          {this.props.dict['header.menu.interface.create']}
        </Button>
        {/* 接口选项 */}
        <Modal
          title={this.props.dict['header.menu.interface.create']}
          visible={this.state.visible}
          width={500}
          maskClosable={false}
          onOk={this.confirmInterface}
          onCancel={() => {this.setState({visible: false})}}
          destroyOnClose
        >
          <MutilForm
            dict={this.props.dict}
            formlist={this.state.formlist}
            wrappedComponentRef={(inst) => this.FormRef = inst}
          />
        </Modal>
      </div>
    )
  }
}

export default CreateFunc
